#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/03/13
#Last Modify: 2018/03/13
# Mastering Machine Learning with Python in Six Steps page 294

import pandas as pd
from sklearn.preprocessing import StandardScaler
from sklearn.cross_validation import train_test_split
from sklearn import cross_validation
from sklearn import metrics

# Bagged Decision Trees for Classification
from sklearn.ensemble import AdaBoostClassifier
from sklearn.tree import DecisionTreeClassifier
# read the data in
path = "~/projects/mastering_machine_learning_python/uci_pima_indians_diabetes/data/pima-data.csv"
df = pd.read_csv(path)
# Let's use some week features to build the tree
X = df[['age','insulin']] # independent variables
y = df['diabetes'].values # dependent variables

#Normalize
X = StandardScaler().fit_transform(X)

# evaluate the model by splitting into train and test sets
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=2017)
kfold = cross_validation.StratifiedKFold(y=y_train, n_folds=5, random_state=2017)

# Dection Tree with 5 fold cross validation
# lets restrict max_depth to 1 to have more impure leaves
clf_DT = DecisionTreeClassifier(max_depth=1,
random_state=2017).fit(X_train,y_train)
results = cross_validation.cross_val_score(clf_DT, X_train,y_train, cv=kfold)
print "Decision Tree (stand alone) - Train : ", results.mean()
print "Decision Tree (stand alone) - Test : ", metrics.accuracy_score(clf_DT.predict(X_test),
                                                                      y_test)
# Using Adaptive Boosting of 100 iteration
num_trees = 100
clf_DT_Boost = AdaBoostClassifier(base_estimator=clf_DT,
                                  n_estimators=num_trees,
                                  learning_rate=0.1,
                                  random_state=2017).fit(X_train,y_train)
results = cross_validation.cross_val_score(clf_DT_Boost, X_train, y_train, cv=kfold)
print "\nDecision Tree (AdaBoosting) - Train : ",results.mean()
print "Decision Tree (AdaBoosting) - Test : ", metrics.accuracy_score(clf_DT_Boost.predict(X_test),
                                                                      y_test)

# page 296
from sklearn.ensemble import GradientBoostingClassifier 
# Using Gradient Boosting of 100 iterations 
clf_GBT = GradientBoostingClassifier(n_estimators=num_trees,
                                     learning_rate=0.1,
                                     random_state=2017).fit(X_train, y_train) 
results = cross_validation.cross_val_score(clf_GBT, X_train, y_train, cv=kfold) 
print "\nGradient Boosting - CV Train : %.2f" % results.mean() 
print "Gradient Boosting - Train : %.2f" % metrics.accuracy_score(clf_GBT.predict(X_train), y_train) 
print "Gradient Boosting - Test : %.2f" % metrics.accuracy_score(clf_GBT.predict(X_test), y_test)


